/*
 * Copyright 2007 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.hibernate4gwt.rebind;

import java.io.PrintWriter;

import net.sf.hibernate4gwt.pojo.ILazyPojo;

import com.google.gwt.core.ext.Generator;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.core.ext.typeinfo.TypeOracle;
import com.google.gwt.user.rebind.ClassSourceFileComposerFactory;
import com.google.gwt.user.rebind.SourceWriter;

/**
 * GWT proxy generator
 * @author bruno.marchesson
 *
 */
public class GwtProxyGenerator extends Generator
{
	
	//-------------------------------------------------------------------------
	//
	// Generator interface
	//
	//-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see com.google.gwt.core.ext.Generator#generate(com.google.gwt.core.ext.TreeLogger, com.google.gwt.core.ext.GeneratorContext, java.lang.String)
	 */
	public String generate(TreeLogger logger, 
						   GeneratorContext context,
						   String typeName) throws UnableToCompleteException
	{
		try
		{
		//	Get target class
		//
			TypeOracle typeOracle = context.getTypeOracle();
			JClassType requestedClass = typeOracle.getType(typeName);
		    String packageName = requestedClass.getPackage().getName();
		    String proxyClassName = ProxyGeneratorHelper.computeProxyName(requestedClass.getSimpleSourceName());
		    String qualifiedProxyClassName = packageName + "." + proxyClassName; 
		    String className = requestedClass.getName(); 
		    
		    logger.log(TreeLogger.INFO,
					   "Generating proxy " + qualifiedProxyClassName + 
					   " for class " + className, 
					   null);
			
		//	Create source writer
		//
			SourceWriter sourceWriter = getSourceWriter(logger, context, packageName, proxyClassName, className);
			if (sourceWriter != null)
			{
				generateILazyPojo(logger, sourceWriter);
				sourceWriter.commit(logger);
				
				logger.log(TreeLogger.INFO, "Proxy generation OK", null);
			}
			return qualifiedProxyClassName;
		}
		catch (Exception ex)
		{
			logger.log(TreeLogger.ERROR, "Proxy generation error", ex);
			throw new UnableToCompleteException();
		}
	}

	//-------------------------------------------------------------------------
	//
	// Internal method
	//
	//-------------------------------------------------------------------------
	/**
	 * Create the needed source writer
	 */
	protected SourceWriter getSourceWriter(TreeLogger logger,
										   GeneratorContext context,
										   String packageName,
										   String className,
										   String superclassName)
	{
		PrintWriter printWriter = context.tryCreate(logger, packageName, className);
		if (printWriter == null)
		{
		//	Could not create print writer (the new type already exists)
		//
			return null;
		}
		
		ClassSourceFileComposerFactory composerFactory = new ClassSourceFileComposerFactory(packageName, className);
		
	//  Add needed import
	//
		composerFactory.addImport("java.util.ArrayList");
		composerFactory.addImport("java.util.List");

	//	Add superclass
	//
		composerFactory.setSuperclass(superclassName);
		
	//	Add ILazyPojo interface and needed import
	//
		composerFactory.addImplementedInterface(ILazyPojo.class.getName());
		
	//	Create source writer
	//
		return composerFactory.createSourceWriter(context, printWriter);
	}
	
	/**
	 * Add lazy properties to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateILazyPojo(TreeLogger logger,
									 SourceWriter sourceWriter)
	{
	//	Generate attribute
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("List of lazy but not loaded properties");
		sourceWriter.println("@gwt.typeArgs <java.lang.String>");
		sourceWriter.endJavaDocComment();
		
		sourceWriter.println("protected List _lazyProperties = new ArrayList();");
		
	//	Generate methods
	//
		generateAddLazyProperty(logger, sourceWriter);
		generateRemoveLazyProperty(logger, sourceWriter);
		generateIsLazyProperty(logger, sourceWriter);
		generateGetLazyString(logger, sourceWriter);
		
		generateToString(logger, sourceWriter);
	}
	
	/**
	 * Generate 'addLazyProperty' method to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateAddLazyProperty(TreeLogger logger,
									 SourceWriter sourceWriter)
	{
	//	Comment
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("Mark the argument property as lazy");
		sourceWriter.endJavaDocComment();
		
	//	Source code
	//
		sourceWriter.println("public void addLazyProperty(String property)");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("_lazyProperties.add(property);");
		sourceWriter.outdent();
		sourceWriter.println("}");
	}
	
	/**
	 * Generate 'removeLazyProperty' method to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateRemoveLazyProperty(TreeLogger logger,
									 		  SourceWriter sourceWriter)
	{
	//	Comment
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("Remove the argument property from lazy list");
		sourceWriter.endJavaDocComment();
		
	//	Source code
	//
		sourceWriter.println("public void removeLazyProperty(String property)");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("_lazyProperties.remove(property);");
		sourceWriter.outdent();
		sourceWriter.println("}");
	}
	
	/**
	 * Generate 'isLazyProperty' method to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateIsLazyProperty(TreeLogger logger,
			 SourceWriter sourceWriter)
	{
	//	Comment
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("Indicates if the argument property is lazy or not");
		sourceWriter.endJavaDocComment();
		
	//	Source code
	//
		sourceWriter.println("public boolean isLazyProperty(String property)");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("return _lazyProperties.contains(property);");
		sourceWriter.outdent();
		sourceWriter.println("}");
	}
	
	/**
	 * Generate 'getLazyString' method to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateGetLazyString(TreeLogger logger,
			 SourceWriter sourceWriter)
	{
	//	Comment
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("Debug method : write the declared lazy properties");
		sourceWriter.endJavaDocComment();
		
	//	Source code
	//
		sourceWriter.println("public String getLazyString()");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("StringBuffer result = new StringBuffer();");
		sourceWriter.println("for (int index = 0 ; index < _lazyProperties.size() ; index ++)");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("result.append(_lazyProperties.get(index));");
		sourceWriter.println("result.append(',');");
		sourceWriter.outdent();
		sourceWriter.println("}");
		sourceWriter.println("return result.toString();");
		sourceWriter.outdent();
		sourceWriter.println("}");
	}
	

	/**
	 * Generate 'toString' method to the instrumented class
	 * @param logger
	 * @param sourceWriter
	 */
	protected void generateToString(TreeLogger logger,
			 					   SourceWriter sourceWriter)
	{
	//	Comment
	//
		sourceWriter.beginJavaDocComment();
		sourceWriter.println("Debug overrides to validate Proxy trasnsformation");
		sourceWriter.endJavaDocComment();
		
	//	Source code
	//
		sourceWriter.println("public String toString()");
		sourceWriter.println("{");
		sourceWriter.indent();
		sourceWriter.println("return \"Proxy with lazy properties : \" + getLazyString();");
		sourceWriter.outdent();
		sourceWriter.println("}");
	}


}
