/*
 * Copyright 2007 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License")
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.sf.hibernate4gwt.rebind;

import javassist.CannotCompileException;
import javassist.ClassClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtField;
import javassist.CtMethod;
import javassist.CtNewMethod;
import net.sf.hibernate4gwt.exception.ProxyException;
import net.sf.hibernate4gwt.pojo.ILazyPojo;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Javassist proxy generator (for server side)
 * @author bruno.marchesson
 *
 */
public class JavassistProxyGenerator implements IServerProxyGenerator
{
	//----
	// Attributes
	//----
	/**
	 * Log channel
	 */
	private static Log log = LogFactory.getLog(JavassistProxyGenerator.class);
	//-------------------------------------------------------------------------
	//
	// Public interface
	//
	//-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see net.sf.hibernate4gwt.rebind.IServerProxyGenerator#generateProxyFor(java.lang.Class)
	 */
	public Class generateProxyFor(Class clazz)
	{
		try
		{
		//	Compute proxy class name
		//
			String sourceClassName = clazz.getName();
			String proxyClassName = ProxyGeneratorHelper.computeProxyName(sourceClassName);
			log.info("Generating server proxy " + proxyClassName + " for class " + sourceClassName);
			
		//	Create proxy class
		//
			ClassPool pool = ClassPool.getDefault();
			
			// TOMCAT and JBOSS classloader handling
			pool.insertClassPath(new ClassClassPath(clazz));
			
			CtClass proxyClass = pool.makeClass(proxyClassName);
			
		//	Add proxy inheritance
		//
			proxyClass.setSuperclass(pool.get(sourceClassName));
			
		//	Add ILazyPojo inheritance
		//
			proxyClass.addInterface(pool.get(ILazyPojo.class.getName()));
			
		//	TODO add lazy properties
		//
			generateILazyPojo(proxyClass);
			
		//	Generate class
		//
			return proxyClass.toClass(clazz.getClassLoader());
		}
		catch(Exception ex)
		{
			throw new ProxyException("Proxy generation failure for " + clazz.getName(), ex);
		}
	}
	
	//-------------------------------------------------------------------------
	//
	// Internal methods
	//
	//-------------------------------------------------------------------------
	/**
	 * Generates ILazyPojo classes and methods
	 * @throws CannotCompileException 
	 */
	private void generateILazyPojo(CtClass proxyClass) throws CannotCompileException
	{
	//	Generate attribute
	//
		CtField lazyPropertiesFied = CtField.make("protected java.util.List _lazyProperties = new java.util.ArrayList();",
												  proxyClass);
		proxyClass.addField(lazyPropertiesFied);
		
	//	Generate methods
	//	
		generateAddLazyProperty(proxyClass);
		generateRemoveLazyProperty(proxyClass);
		generateIsLazyProperty(proxyClass);
	}	
	
	/**
	 * Generate 'addLazyProperty' method to the instrumented class
	 * @param proxyClass
	 * @throws CannotCompileException 
	 */
	private void generateAddLazyProperty(CtClass proxyClass) throws CannotCompileException
	{
	//	Source code
	//
		StringBuffer sourceCode = new StringBuffer();
		sourceCode.append("public void addLazyProperty(String property)");
		sourceCode.append("{ _lazyProperties.add(property); }");
		
	//	Add method body
	//
		CtMethod method = CtNewMethod.make(sourceCode.toString(), proxyClass);
		proxyClass.addMethod(method);
		
	}
	
	/**
	 * Generate 'addLazyProperty' method to the instrumented class
	 * @param proxyClass
	 * @throws CannotCompileException 
	 */
	private void generateRemoveLazyProperty(CtClass proxyClass) throws CannotCompileException
	{
	//	Source code
	//
		StringBuffer sourceCode = new StringBuffer();
		sourceCode.append("public void removeLazyProperty(String property)");
		sourceCode.append("{ _lazyProperties.remove(property); }");
		
	//	Add method body
	//
		CtMethod method = CtNewMethod.make(sourceCode.toString(), proxyClass);
		proxyClass.addMethod(method);	
	}
	
	/**
	 * Generate 'isLazyProperty' method to the instrumented class
	 * @param proxyClass
	 * @throws CannotCompileException 
	 */
	private void generateIsLazyProperty(CtClass proxyClass) throws CannotCompileException
	{
	//	Source code
	//
		StringBuffer sourceCode = new StringBuffer();
		sourceCode.append("public boolean isLazyProperty(String property)");
		sourceCode.append("{ return _lazyProperties.contains(property); }");
		
	//	Add method body
	//
		CtMethod method = CtNewMethod.make(sourceCode.toString(), proxyClass);
		proxyClass.addMethod(method);
	}
	
	/**
	 * Generate 'getLazyString' method to the instrumented class
	 * @throws CannotCompileException 
	 */
	protected void generateGetLazyString(CtClass proxyClass) throws CannotCompileException
	{
	//	Source code
	//
		StringBuffer sourceCode = new StringBuffer();
		sourceCode.append("public String getLazyString()");
		sourceCode.append("{");
		sourceCode.append("StringBuffer result = new StringBuffer();");
		sourceCode.append("for (int index = 0 ; index < _lazyProperties.size() ; index ++)");
		sourceCode.append("{");
		sourceCode.append("result.append(_lazyProperties.get(index));");
		sourceCode.append("result.append(',');");
		sourceCode.append("}");
		sourceCode.append("return result.toString();");
		sourceCode.append("}");
		
	//	Add method body
	//
		CtMethod method = CtNewMethod.make(sourceCode.toString(), proxyClass);
		proxyClass.addMethod(method);
	}
}
